@@ -2,24 +2,37 @@
 
 extern crate test;
 
-use rand::{thread_rng, Rng};
-use solana_core::{
-    broadcast_stage::{broadcast_metrics::TransmitShredsStats, broadcast_shreds, BroadcastStage},
-    cluster_nodes::ClusterNodes,
+use {
+    rand::{thread_rng, Rng},
+    solana_core::{
+        broadcast_stage::{
+            broadcast_metrics::TransmitShredsStats, broadcast_shreds, BroadcastStage,
+        },
+        cluster_nodes::ClusterNodesCache,
+    },
+    solana_gossip::{
+        cluster_info::{ClusterInfo, Node},
+        contact_info::ContactInfo,
+    },
+    solana_ledger::{
+        genesis_utils::{create_genesis_config, GenesisConfigInfo},
+        shred::Shred,
+    },
+    solana_runtime::{bank::Bank, bank_forks::BankForks},
+    solana_sdk::{
+        pubkey,
+        signature::Keypair,
+        timing::{timestamp, AtomicInterval},
+    },
+    solana_streamer::socket::SocketAddrSpace,
+    std::{
+        collections::HashMap,
+        net::UdpSocket,
+        sync::{Arc, RwLock},
+        time::Duration,
+    },
+    test::Bencher,
 };
-use solana_gossip::{
-    cluster_info::{ClusterInfo, Node},
-    contact_info::ContactInfo,
-};
-use solana_ledger::shred::Shred;
-use solana_sdk::{
-    pubkey,
-    signature::Keypair,
-    timing::{timestamp, AtomicInterval},
-};
-use solana_streamer::socket::SocketAddrSpace;
-use std::{collections::HashMap, net::UdpSocket, sync::Arc};
-use test::Bencher;
 
 #[bench]
 fn broadcast_shreds_bench(bencher: &mut Bencher) {
@@ -33,6 +46,10 @@ fn broadcast_shreds_bench(bencher: &mut Bencher) {
     );
     let socket = UdpSocket::bind("0.0.0.0:0").unwrap();
 
+    let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
+    let bank = Bank::new(&genesis_config);
+    let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
+
     const NUM_SHREDS: usize = 32;
     let shreds = vec![Shred::new_empty_data_shred(); NUM_SHREDS];
     let mut stakes = HashMap::new();
@@ -44,7 +61,10 @@ fn broadcast_shreds_bench(bencher: &mut Bencher) {
         stakes.insert(id, thread_rng().gen_range(1, NUM_PEERS) as u64);
     }
     let cluster_info = Arc::new(cluster_info);
-    let cluster_nodes = ClusterNodes::<BroadcastStage>::new(&cluster_info, &stakes);
+    let cluster_nodes_cache = ClusterNodesCache::<BroadcastStage>::new(
+        8,                      // cap
+        Duration::from_secs(5), // ttl
+    );
     let shreds = Arc::new(shreds);
     let last_datapoint = Arc::new(AtomicInterval::default());
     bencher.iter(move || {
@@ -52,10 +72,12 @@ fn broadcast_shreds_bench(bencher: &mut Bencher) {
         broadcast_shreds(
             &socket,
             &shreds,
-            &cluster_nodes,
+            &cluster_nodes_cache,
             &last_datapoint,
             &mut TransmitShredsStats::default(),
             &SocketAddrSpace::Unspecified,
+            &cluster_info,
+            &bank_forks,
         )
         .unwrap();
     });
