@@ -1,36 +1,42 @@
 //! A stage to broadcast data from a leader node to validators
 #![allow(clippy::rc_buffer)]
-use self::{
-    broadcast_duplicates_run::BroadcastDuplicatesRun,
-    broadcast_fake_shreds_run::BroadcastFakeShredsRun, broadcast_metrics::*,
-    fail_entry_verification_broadcast_run::FailEntryVerificationBroadcastRun,
-    standard_broadcast_run::StandardBroadcastRun,
-};
-use crate::{
-    cluster_nodes::ClusterNodes,
-    result::{Error, Result},
-};
-use crossbeam_channel::{
-    Receiver as CrossbeamReceiver, RecvTimeoutError as CrossbeamRecvTimeoutError,
-    Sender as CrossbeamSender,
-};
-use solana_gossip::cluster_info::{ClusterInfo, ClusterInfoError};
-use solana_ledger::{blockstore::Blockstore, shred::Shred};
-use solana_measure::measure::Measure;
-use solana_metrics::{inc_new_counter_error, inc_new_counter_info};
-use solana_poh::poh_recorder::WorkingBankEntry;
-use solana_runtime::bank::Bank;
-use solana_sdk::timing::{timestamp, AtomicInterval};
-use solana_sdk::{clock::Slot, pubkey::Pubkey};
-use solana_streamer::{sendmmsg::send_mmsg, socket::SocketAddrSpace};
-use std::{
-    collections::HashMap,
-    net::UdpSocket,
-    sync::atomic::{AtomicBool, Ordering},
-    sync::mpsc::{channel, Receiver, RecvError, RecvTimeoutError, Sender},
-    sync::{Arc, Mutex},
-    thread::{self, Builder, JoinHandle},
-    time::{Duration, Instant},
+use {
+    self::{
+        broadcast_duplicates_run::BroadcastDuplicatesRun,
+        broadcast_fake_shreds_run::BroadcastFakeShredsRun, broadcast_metrics::*,
+        fail_entry_verification_broadcast_run::FailEntryVerificationBroadcastRun,
+        standard_broadcast_run::StandardBroadcastRun,
+    },
+    crate::{
+        cluster_nodes::{ClusterNodes, ClusterNodesCache},
+        result::{Error, Result},
+    },
+    crossbeam_channel::{
+        Receiver as CrossbeamReceiver, RecvTimeoutError as CrossbeamRecvTimeoutError,
+        Sender as CrossbeamSender,
+    },
+    itertools::Itertools,
+    solana_gossip::cluster_info::{ClusterInfo, ClusterInfoError},
+    solana_ledger::{blockstore::Blockstore, shred::Shred},
+    solana_measure::measure::Measure,
+    solana_metrics::{inc_new_counter_error, inc_new_counter_info},
+    solana_poh::poh_recorder::WorkingBankEntry,
+    solana_runtime::{bank::Bank, bank_forks::BankForks},
+    solana_sdk::timing::{timestamp, AtomicInterval},
+    solana_sdk::{clock::Slot, pubkey::Pubkey},
+    solana_streamer::{
+        sendmmsg::{batch_send, SendPktsError},
+        socket::SocketAddrSpace,
+    },
+    std::{
+        collections::HashMap,
+        net::UdpSocket,
+        sync::atomic::{AtomicBool, Ordering},
+        sync::mpsc::{channel, Receiver, RecvError, RecvTimeoutError, Sender},
+        sync::{Arc, Mutex, RwLock},
+        thread::{self, Builder, JoinHandle},
+        time::{Duration, Instant},
+    },
 };
 
 mod broadcast_duplicates_run;
@@ -40,11 +46,14 @@ pub(crate) mod broadcast_utils;
 mod fail_entry_verification_broadcast_run;
 mod standard_broadcast_run;
 
+const CLUSTER_NODES_CACHE_NUM_EPOCH_CAP: usize = 8;
+const CLUSTER_NODES_CACHE_TTL: Duration = Duration::from_secs(5);
+
 pub(crate) const NUM_INSERT_THREADS: usize = 2;
 pub(crate) type RetransmitSlotsSender = CrossbeamSender<HashMap<Slot, Arc<Bank>>>;
 pub(crate) type RetransmitSlotsReceiver = CrossbeamReceiver<HashMap<Slot, Arc<Bank>>>;
 pub(crate) type RecordReceiver = Receiver<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>;
-pub(crate) type TransmitReceiver = Receiver<(TransmitShreds, Option<BroadcastShredBatchInfo>)>;
+pub(crate) type TransmitReceiver = Receiver<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>;
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub enum BroadcastStageReturnType {
@@ -68,6 +77,7 @@ pub enum BroadcastStageType {
 }
 
 impl BroadcastStageType {
+    #[allow(clippy::too_many_arguments)]
     pub fn new_broadcast_stage(
         &self,
         sock: Vec<UdpSocket>,
@@ -76,6 +86,7 @@ impl BroadcastStageType {
         retransmit_slots_receiver: RetransmitSlotsReceiver,
         exit_sender: &Arc<AtomicBool>,
         blockstore: &Arc<Blockstore>,
+        bank_forks: &Arc<RwLock<BankForks>>,
         shred_version: u16,
     ) -> BroadcastStage {
         let keypair = cluster_info.keypair.clone();
@@ -87,6 +98,7 @@ impl BroadcastStageType {
                 retransmit_slots_receiver,
                 exit_sender,
                 blockstore,
+                bank_forks,
                 StandardBroadcastRun::new(keypair, shred_version),
             ),
 
@@ -97,6 +109,7 @@ impl BroadcastStageType {
                 retransmit_slots_receiver,
                 exit_sender,
                 blockstore,
+                bank_forks,
                 FailEntryVerificationBroadcastRun::new(keypair, shred_version),
             ),
 
@@ -107,6 +120,7 @@ impl BroadcastStageType {
                 retransmit_slots_receiver,
                 exit_sender,
                 blockstore,
+                bank_forks,
                 BroadcastFakeShredsRun::new(keypair, 0, shred_version),
             ),
 
@@ -117,19 +131,19 @@ impl BroadcastStageType {
                 retransmit_slots_receiver,
                 exit_sender,
                 blockstore,
+                bank_forks,
                 BroadcastDuplicatesRun::new(keypair, shred_version, config.clone()),
             ),
         }
     }
 }
 
-pub type TransmitShreds = (Option<Arc<HashMap<Pubkey, u64>>>, Arc<Vec<Shred>>);
 trait BroadcastRun {
     fn run(
         &mut self,
         blockstore: &Arc<Blockstore>,
         receiver: &Receiver<WorkingBankEntry>,
-        socket_sender: &Sender<(TransmitShreds, Option<BroadcastShredBatchInfo>)>,
+        socket_sender: &Sender<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>,
         blockstore_sender: &Sender<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>,
     ) -> Result<()>;
     fn transmit(
@@ -137,6 +151,7 @@ trait BroadcastRun {
         receiver: &Arc<Mutex<TransmitReceiver>>,
         cluster_info: &ClusterInfo,
         sock: &UdpSocket,
+        bank_forks: &Arc<RwLock<BankForks>>,
     ) -> Result<()>;
     fn record(
         &mut self,
@@ -172,7 +187,7 @@ impl BroadcastStage {
     fn run(
         blockstore: &Arc<Blockstore>,
         receiver: &Receiver<WorkingBankEntry>,
-        socket_sender: &Sender<(TransmitShreds, Option<BroadcastShredBatchInfo>)>,
+        socket_sender: &Sender<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>,
         blockstore_sender: &Sender<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>,
         mut broadcast_stage_run: impl BroadcastRun,
     ) -> BroadcastStageReturnType {
@@ -230,6 +245,7 @@ impl BroadcastStage {
         retransmit_slots_receiver: RetransmitSlotsReceiver,
         exit_sender: &Arc<AtomicBool>,
         blockstore: &Arc<Blockstore>,
+        bank_forks: &Arc<RwLock<BankForks>>,
         broadcast_stage_run: impl BroadcastRun + Send + 'static + Clone,
     ) -> Self {
         let btree = blockstore.clone();
@@ -258,10 +274,12 @@ impl BroadcastStage {
             let socket_receiver = socket_receiver.clone();
             let mut bs_transmit = broadcast_stage_run.clone();
             let cluster_info = cluster_info.clone();
+            let bank_forks = bank_forks.clone();
             let t = Builder::new()
                 .name("solana-broadcaster-transmit".to_string())
                 .spawn(move || loop {
-                    let res = bs_transmit.transmit(&socket_receiver, &cluster_info, &sock);
+                    let res =
+                        bs_transmit.transmit(&socket_receiver, &cluster_info, &sock, &bank_forks);
                     let res = Self::handle_error(res, "solana-broadcaster-transmit");
                     if let Some(res) = res {
                         return res;
@@ -312,7 +330,7 @@ impl BroadcastStage {
     fn check_retransmit_signals(
         blockstore: &Blockstore,
         retransmit_slots_receiver: &RetransmitSlotsReceiver,
-        socket_sender: &Sender<(TransmitShreds, Option<BroadcastShredBatchInfo>)>,
+        socket_sender: &Sender<(Arc<Vec<Shred>>, Option<BroadcastShredBatchInfo>)>,
     ) -> Result<()> {
         let timer = Duration::from_millis(100);
 
@@ -323,27 +341,26 @@ impl BroadcastStage {
         }
 
         for (_, bank) in retransmit_slots.iter() {
-            let bank_epoch = bank.get_leader_schedule_epoch(bank.slot());
-            let stakes = bank.epoch_staked_nodes(bank_epoch);
-            let stakes = stakes.map(Arc::new);
+            let slot = bank.slot();
             let data_shreds = Arc::new(
                 blockstore
-                    .get_data_shreds_for_slot(bank.slot(), 0)
+                    .get_data_shreds_for_slot(slot, 0)
                     .expect("My own shreds must be reconstructable"),
             );
-
+            debug_assert!(data_shreds.iter().all(|shred| shred.slot() == slot));
             if !data_shreds.is_empty() {
-                socket_sender.send(((stakes.clone(), data_shreds), None))?;
+                socket_sender.send((data_shreds, None))?;
             }
 
             let coding_shreds = Arc::new(
                 blockstore
-                    .get_coding_shreds_for_slot(bank.slot(), 0)
+                    .get_coding_shreds_for_slot(slot, 0)
                     .expect("My own shreds must be reconstructable"),
             );
 
+            debug_assert!(coding_shreds.iter().all(|shred| shred.slot() == slot));
             if !coding_shreds.is_empty() {
-                socket_sender.send(((stakes.clone(), coding_shreds), None))?;
+                socket_sender.send((coding_shreds, None))?;
             }
         }
 
@@ -359,11 +376,13 @@ impl BroadcastStage {
 }
 
 fn update_peer_stats(
-    num_live_peers: i64,
-    broadcast_len: i64,
+    cluster_nodes: &ClusterNodes<BroadcastStage>,
     last_datapoint_submit: &Arc<AtomicInterval>,
 ) {
     if last_datapoint_submit.should_update(1000) {
+        let now = timestamp();
+        let num_live_peers = cluster_nodes.num_peers_live(now);
+        let broadcast_len = cluster_nodes.num_peers() + 1;
         datapoint_info!(
             "cluster_info-num_nodes",
             ("live_count", num_live_peers, i64),
@@ -377,51 +396,51 @@ fn update_peer_stats(
 pub fn broadcast_shreds(
     s: &UdpSocket,
     shreds: &[Shred],
-    cluster_nodes: &ClusterNodes<BroadcastStage>,
+    cluster_nodes_cache: &ClusterNodesCache<BroadcastStage>,
     last_datapoint_submit: &Arc<AtomicInterval>,
     transmit_stats: &mut TransmitShredsStats,
     socket_addr_space: &SocketAddrSpace,
+    cluster_info: &ClusterInfo,
+    bank_forks: &Arc<RwLock<BankForks>>,
 ) -> Result<()> {
-    let broadcast_len = cluster_nodes.num_peers();
-    if broadcast_len == 0 {
-        update_peer_stats(1, 1, last_datapoint_submit);
-        return Ok(());
-    }
+    let mut result = Ok(());
     let mut shred_select = Measure::start("shred_select");
+    // Only the leader broadcasts shreds.
+    let leader = cluster_info.id();
+    let (root_bank, working_bank) = {
+        let bank_forks = bank_forks.read().unwrap();
+        (bank_forks.root_bank(), bank_forks.working_bank())
+    };
     let packets: Vec<_> = shreds
         .iter()
-        .filter_map(|shred| {
-            let node = cluster_nodes.get_broadcast_peer(shred.seed())?;
-            if socket_addr_space.check(&node.tvu) {
-                Some((&shred.payload, &node.tvu))
-            } else {
-                None
-            }
+        .group_by(|shred| shred.slot())
+        .into_iter()
+        .flat_map(|(slot, shreds)| {
+            let cluster_nodes =
+                cluster_nodes_cache.get(slot, &root_bank, &working_bank, cluster_info);
+            update_peer_stats(&cluster_nodes, last_datapoint_submit);
+            let root_bank = root_bank.clone();
+            shreds.filter_map(move |shred| {
+                let seed = shred.seed(leader, &root_bank);
+                let node = cluster_nodes.get_broadcast_peer(seed)?;
+                socket_addr_space
+                    .check(&node.tvu)
+                    .then(|| (&shred.payload, node.tvu))
+            })
         })
         .collect();
     shred_select.stop();
     transmit_stats.shred_select += shred_select.as_us();
 
-    let mut sent = 0;
     let mut send_mmsg_time = Measure::start("send_mmsg");
-    while sent < packets.len() {
-        match send_mmsg(s, &packets[sent..]) {
-            Ok(n) => sent += n,
-            Err(e) => {
-                return Err(Error::Io(e));
-            }
-        }
+    if let Err(SendPktsError::IoError(ioerr, num_failed)) = batch_send(s, &packets[..]) {
+        transmit_stats.dropped_packets += num_failed;
+        result = Err(Error::Io(ioerr));
     }
     send_mmsg_time.stop();
     transmit_stats.send_mmsg_elapsed += send_mmsg_time.as_us();
-
-    let num_live_peers = cluster_nodes.num_peers_live(timestamp()) as i64;
-    update_peer_stats(
-        num_live_peers,
-        broadcast_len as i64 + 1,
-        last_datapoint_submit,
-    );
-    Ok(())
+    transmit_stats.total_packets += packets.len();
+    result
 }
 
 #[cfg(test)]
@@ -447,15 +466,15 @@ pub mod test {
     };
 
     #[allow(clippy::implicit_hasher)]
-    pub fn make_transmit_shreds(
+    #[allow(clippy::type_complexity)]
+    fn make_transmit_shreds(
         slot: Slot,
         num: u64,
-        stakes: Option<Arc<HashMap<Pubkey, u64>>>,
     ) -> (
         Vec<Shred>,
         Vec<Shred>,
-        Vec<TransmitShreds>,
-        Vec<TransmitShreds>,
+        Vec<Arc<Vec<Shred>>>,
+        Vec<Arc<Vec<Shred>>>,
     ) {
         let num_entries = max_ticks_per_n_shreds(num, None);
         let (data_shreds, _) = make_slot_entries(slot, 0, num_entries);
@@ -472,11 +491,11 @@ pub mod test {
             coding_shreds.clone(),
             data_shreds
                 .into_iter()
-                .map(|s| (stakes.clone(), Arc::new(vec![s])))
+                .map(|shred| Arc::new(vec![shred]))
                 .collect(),
             coding_shreds
                 .into_iter()
-                .map(|s| (stakes.clone(), Arc::new(vec![s])))
+                .map(|shred| Arc::new(vec![shred]))
                 .collect(),
         )
     }
@@ -488,15 +507,15 @@ pub mod test {
         num_expected_data_shreds: u64,
         num_expected_coding_shreds: u64,
     ) {
-        while let Ok((new_retransmit_slots, _)) = transmit_receiver.try_recv() {
-            if new_retransmit_slots.1[0].is_data() {
-                for data_shred in new_retransmit_slots.1.iter() {
+        while let Ok((shreds, _)) = transmit_receiver.try_recv() {
+            if shreds[0].is_data() {
+                for data_shred in shreds.iter() {
                     assert_eq!(data_shred.index() as u64, data_index);
                     data_index += 1;
                 }
             } else {
-                assert_eq!(new_retransmit_slots.1[0].index() as u64, coding_index);
-                for coding_shred in new_retransmit_slots.1.iter() {
+                assert_eq!(shreds[0].index() as u64, coding_index);
+                for coding_shred in shreds.iter() {
                     assert_eq!(coding_shred.index() as u64, coding_index);
                     coding_index += 1;
                 }
@@ -520,7 +539,7 @@ pub mod test {
         // Make some shreds
         let updated_slot = 0;
         let (all_data_shreds, all_coding_shreds, _, _all_coding_transmit_shreds) =
-            make_transmit_shreds(updated_slot, 10, None);
+            make_transmit_shreds(updated_slot, 10);
         let num_data_shreds = all_data_shreds.len();
         let num_coding_shreds = all_coding_shreds.len();
         assert!(num_data_shreds >= 10);
@@ -591,7 +610,9 @@ pub mod test {
         let exit_sender = Arc::new(AtomicBool::new(false));
 
         let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
-        let bank = Arc::new(Bank::new(&genesis_config));
+        let bank = Bank::new(&genesis_config);
+        let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
+        let bank = bank_forks.read().unwrap().root_bank();
 
         let leader_keypair = cluster_info.keypair.clone();
         // Start up the broadcast stage
@@ -602,6 +623,7 @@ pub mod test {
             retransmit_slots_receiver,
             &exit_sender,
             &blockstore,
+            &bank_forks,
             StandardBroadcastRun::new(leader_keypair, 0),
         );
 
