@@ -1,6 +1,7 @@
 use bincode::serialize;
 use jsonrpc_core::futures::StreamExt;
 use jsonrpc_core_client::transports::ws;
+
 use log::*;
 use reqwest::{self, header::CONTENT_TYPE};
 use serde_json::{json, Value};
@@ -10,11 +11,12 @@ use solana_client::{
     rpc_client::RpcClient,
     rpc_config::{RpcAccountInfoConfig, RpcSignatureSubscribeConfig},
     rpc_request::RpcError,
-    rpc_response::{Response, RpcSignatureResult, SlotUpdate},
+    rpc_response::{Response as RpcResponse, RpcSignatureResult, SlotUpdate},
     tpu_client::{TpuClient, TpuClientConfig},
 };
 use solana_core::test_validator::TestValidator;
 use solana_rpc::rpc_pubsub::gen_client::Client as PubsubClient;
+
 use solana_sdk::{
     commitment_config::CommitmentConfig,
     hash::Hash,
@@ -256,9 +258,9 @@ fn test_rpc_subscriptions() {
     // Track when subscriptions are ready
     let (ready_sender, ready_receiver) = channel::<()>();
     // Track account notifications are received
-    let (account_sender, account_receiver) = channel::<Response<UiAccount>>();
+    let (account_sender, account_receiver) = channel::<RpcResponse<UiAccount>>();
     // Track when status notifications are received
-    let (status_sender, status_receiver) = channel::<(String, Response<RpcSignatureResult>)>();
+    let (status_sender, status_receiver) = channel::<(String, RpcResponse<RpcSignatureResult>)>();
 
     // Create the pub sub runtime
     let rt = Runtime::new().unwrap();
